/* ---------------------------------------------------------------
Thu May 18 15:30:13 MET DST 2000

simple tool for screen scrapper SNMP agent 
agent is in TCL, scotty implementation 
generate output in plain shellable/awkable form
or in tcl list oriented form

--------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <syslog.h>

#include <kstat.h>
#include <sys/envctrl_gen.h>

#include "envstat.h"

void print_ps(kstat_ctl_t *kc);
void print_fan(kstat_ctl_t *kc);
void print_disk(kstat_ctl_t *kc);

/*
global to control type of output 
values are defined in envstat.h file
*/
int output=SIMPLE_OUT;
/* 
global to control sysloging, 0 means no logging
*/
int tolog=0;
/*
global output buffer ...
*/
char outbuff[1024];


/* ---------------------------------------------------- */

/* 
print the usage message 
*/
void usage( char *cmd ) {
   (void)fprintf(
	stderr, 
	"usage: %s -l [-v | -d -f -t] [-o tcl] \n", 
	cmd
	);
   exit	(2);
}

/*
does the actual output, it is done in this way 
because of single point of sysloging
if you want to change syslog message, level you can done it here!
*/
void myOut(char *bf) {
	if (bf && *bf) {
		puts(bf);
		if(tolog) { syslog(LOG_ALERT,bf); }
	}
}

/*
general trailer print 
*/
void print_trailer( void ) {
	if (output!=SIMPLE_OUT) {
		sprintf (outbuff,"}");
	} else {
		sprintf (outbuff,"");
	}
	myOut(outbuff);
}

/*  ------------------------------------------------
options :
-v --> verbose
-l --> to syslog
-o  format = tcl perl 
-t  temperature
-f fan
-d disks
 ------------------------------------------------ */

main (int argc, char **argv)
{
int verbose=0;	/* -d -f -t options all together */
int dodisk=0;	/* print data for disks only */
int dofan=0;	/* print data for fans only */
int dotemp=0;	/* print data for temperature only */

int c;			/* argument pasrsing .... */
extern char *optarg;	/* argument pasrsing .... */ 
extern int optind	/* argument pasrsing .... */;

char *ofile=NULL	/* argument pasrsing .... */;
int errflg=0		/* argument pasrsing .... */;

kstat_ctl_t *kc;	/* kernel chain prt */
kstat_t *ksp;		/* kstat structure prt */


/*
parse command line 
*/
	while ((c = getopt(argc, argv, "vltfdo:")) != EOF)
	   switch (c) {
	   case	'v':
		if (verbose) { errflg++; }
		verbose=dodisk=dofan=dotemp=1; /* print out all values */
	      break;
	   case 'd':
		if (dodisk) { errflg++;}
		dodisk=1; 
		break;
	   case 'f':
		if (dofan) { errflg++;}
		dofan=1; 
		break;
	   case 't':
		if (dotemp) { errflg++;}
		dotemp=1; 
		break;
	   case	'l':
		tolog=1;	/* send to syslog too */
	      break;
	   case	'o':		/* define the output type */
		if (!(strcmp(optarg,"tcl"))) { output=TCL_OUT; }
	      break;
	   case	'?':
	      errflg++;
	   }


if (errflg) {
	usage(argv[0]);	/* print message and out */
}

	outbuff[0]='\0';


	kc=kstat_open();	/* open the kstat chain */
	if(kc){
	/* 
		get the env control module 
		-1 means any module by nunmber
		NULL, means any module by name 

		there is some theory in readme file
	*/

	ksp=kstat_lookup(kc,ENVCTRL_MODULE_NAME,-1,NULL);

	if(ksp){                        

		/* walk trough module and print values */

		if (dotemp) { print_temp(kc); }
		if (dodisk) { print_disk(kc); }
		if (dofan) { print_fan(kc); }
	}

	kstat_close(kc);
	exit(0);
	}
}

/* end */
