/*------------------------------------------------------------*
|                                                             |
| Interface for tcpd wrapper fucntions, for scotty agent      |
| Author:	Damir Delija                                  |
|               University Computing Center, SRCE             |
| Date:         Mon May 28 2001                               |
|               2001,  Damir Delija                           |
|                                                             |
*--------------------Procedure Description--------------------*
|                                                             |
| based on W.Wenema tcpd system                               |
|                                                             |
| hosts_ctl()  is  a  wrapper  around                         |
| the request_init() and hosts_access() routines              |
| with  a  perhaps  more  convenient interface                |
| (though  it does not pass on enough information             |
| to support automated client username lookups).              |
|                                                             |
| The client host  address,  client  host  name               |
| and username arguments should contain valid data            |
| or STRING_UNKNOWN.   hosts_ctl() returns zero if            |
| access should be denied.  The  allow_severity               |
| and deny_severity variables determine how                   |
| accepted and rejected requests  may  be  logged.            |
|                                                             |
| They must  be  provided  by  the  caller and                |
| may be modified by rules in the access control              |
| tables.                                                     |
|                                                             |
| hosts_ctl() returns zero if access should be denied.        |
|                                                             |
*----------------Modification Audit/Change Log----------------*
| Rev  Sar                                                    |
| No   Code  Date      Rellvl  Mod  Description               |
| --   ----- --------  ------  ---  --------------------------|
|                                                             |
*-------------------------------------------------------------*/

#include <stddef.h>
#include <string.h>

#include <stdlib.h>

#include <tcl.h>


int hosts_ctl(
	char *daemon,
	char *client_name,
	char *client_addr,
	char *client_user
);

#ifndef STRING_UNKNOWN	
#define STRING_UNKNOWN	"unknown"
#endif

int allow_severity=0;
int deny_severity=0;


int tcl_hosts_ctl(
	void *clientData, 
	Tcl_Interp *interp, 
	int argc, 
	char **argv)
{
int r;
int i;			/* counter */
char *iARGV[5];		
			/* array for arguments, easy grouped 
			   deamon client_name client_addr client_user
			*/

char *UNKNOWN=STRING_UNKNOWN;

/* parse input , and check numeber of inputs */

	if (argc < 4) {
		Tcl_AppendResult( interp, 
		"bad # argv :", argv[0], 
		" daemon client_name client_addr client_user ",
		(char*) NULL);
		return (TCL_ERROR);
	}

/* if any of args is {} set it to UNKNOWN */

	for(i=1; i<5; i++) {
		if ( argv[i] && *argv[i] ) { 
			iARGV[i]=argv[i];
		} else {
			iARGV[i]=UNKNOWN;
		}
	}

	r=hosts_ctl( iARGV[1], iARGV[2], iARGV[3], iARGV[4] ) ;

/*
printf ( "rez= %d \n",r);
*/

	if ( r ) {
 		sprintf (interp->result,"1");
		return (TCL_OK );
	}
	return (TCL_ERROR);
}


/* 
 * Initialize the package, with Tcl interpreter
*/

int Tcpd_Init( Tcl_Interp *interp ) 
{

	Tcl_CreateCommand( 	
		interp, 		
 		"hosts_ctl", tcl_hosts_ctl,
		(ClientData)NULL, (Tcl_CmdDeleteProc *)NULL 
	);
	return (TCL_OK);

}

/*
end
*/
